package com.nisum.example.controller;

import com.nisum.example.model.Reservation;
import com.nisum.example.service.ReservationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

@RestController
@RequestMapping(ReservationController.ROOM_V_1_RESERVATION) //base string of url
@CrossOrigin //to run angular and spring app locally
public class ReservationController {

    public static final String ROOM_V_1_RESERVATION = "/room/v1/reservation/";

    private final ReservationService reservationService;

    @Autowired
    public ReservationController(ReservationService reservationService) {
        this.reservationService = reservationService;
    }

    @GetMapping(path = "{id}",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Mono<Reservation> getReservationById(@PathVariable String id){
        return reservationService.getReservation(id);
    }


    @GetMapping(path = "all",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Flux<Reservation> getAllReservations(){
        return reservationService.getAllReservations();
    }


    @PostMapping(path = "",produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
    consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Mono<Reservation> createReservation(@RequestBody Mono<Reservation> reservation){
             return reservationService.createReservation(reservation);
    }

    @PostMapping(path = "reservations",produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
            consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Flux<Reservation> createReservations(@RequestBody List<Reservation> reservationList){
        return reservationService.createReservations(reservationList);
    }
    @PutMapping(path = "{id}",produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
            consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Mono<Reservation> updatePrice(@PathVariable String id,
                                    @RequestBody Mono<Reservation> reservation) {
        return reservationService.updateReservation(id,reservation);
    }

    @DeleteMapping(path = "{id}")
    public Mono<Boolean> deleteReservation(@PathVariable String id){
        return reservationService.deleteReservation(id);

    }
}
