package com.nisum.example.controller;

import com.nisum.example.model.Reservation;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.ApplicationContext;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Mono;

import java.time.LocalDate;
import java.time.temporal.ChronoUnit;

@ExtendWith(SpringExtension.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
class ReservationControllerTest {

    @Autowired
    private ApplicationContext context;
    private WebTestClient webTestClient;
    private Reservation reservation;


    @BeforeEach
    void setUp() {
        webTestClient=WebTestClient.bindToApplicationContext(this.context)
                .build();
        reservation=new
                Reservation(123L, LocalDate.now(),
                LocalDate.now().plus(10, ChronoUnit.DAYS),150);
    }

    @Test
    void getReservation() {
        webTestClient.get().uri("/room/v1/reservation/123")
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_UTF8)
                .expectBodyList(Reservation.class);
    }

    @Test
    void getAllReservations() {
        webTestClient.get().uri("/room/v1/reservation/all")
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_UTF8)
                .expectBodyList(Reservation.class);
    }

    @Test
    void createReservation() {
        webTestClient.post()
                .uri("/room/v1/reservation/")
                .body(Mono.just(reservation),Reservation.class)
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_UTF8)
                .expectBody(Reservation.class);
    }

    /*@Test
    void createReservations() {
        webTestClient.post()
                .uri("/room/v1/reservation/reservations")
                .body(Flux.just(reservation),Reservation.class)
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_UTF8)
                .expectBody(Reservation.class);
    }*/
}